﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Security;
using System.Text;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace JustAddCode.EffectiveFeatures
{
    public class FeatureGroupActivation : WebPart
    {
        RadioButtonList radioButtons = null;

        private SPFarm CurrentFarm
        {
            get
            {
                return Microsoft.SharePoint.Administration.SPFarm.Local;
            }
        }

        private SPWeb CurrentWeb
        {
            get
            {
                return SPContext.Current.Web;
            }
        }

        protected override void CreateChildControls()
        {
            base.CreateChildControls();

            // Features with a Group property
            var features = from SPFeatureDefinition f in CurrentFarm.FeatureDefinitions
                           where f.Scope == SPFeatureScope.Web 
                           && (from SPFeatureProperty prop in f.Properties
                                   where prop.Name == "Group"
                                   select prop).Count() > 0
                           select f;


            //List<string> groups = new List<string>();
            Hashtable groups = new Hashtable();


            foreach (var feature in features)
            {
                SPFeatureProperty prop = feature.Properties["Group"];
                string group = prop.Value;
                if(!groups.Contains(group))
                    groups.Add(group,1);
                else
                {
                    groups[group] = (int) groups[group] + 1;
                }
                
            }
            
            radioButtons = new RadioButtonList();

            foreach (string group in (from string k in groups.Keys orderby(k) select k))
            {
                
                ListItem rb = new ListItem();
                rb.Value = group;
                rb.Text = string.Format("{0} - ({1})",group,groups[group]);
                radioButtons.Items.Add(rb);
            }

            Controls.Add(radioButtons);

            Button btnActivate = new Button();
            btnActivate.Text = "Activate!";
            btnActivate.Click += new EventHandler(btnActivate_Click);
            Controls.Add(btnActivate);

            Button btnDeactivate = new Button();
            btnDeactivate.Text = "Dectivate!";
            btnDeactivate.Click += new EventHandler(btnDeactivate_Click);
            Controls.Add(btnDeactivate);

        }

        void btnActivate_Click(object sender, EventArgs e)
        {
            Label l = new Label();
            l.Text = "Activating Group: " + radioButtons.SelectedItem.Value + ": ";

            foreach (SPFeatureDefinition definition in FindFeaturesByGroup(radioButtons.SelectedItem.Value))
            {
                if(!CurrentWeb.Features.Contains(definition.Id))
                {
                    CurrentWeb.Features.Add(definition.Id);
                    l.Text += definition.DisplayName + ",";
                }
            }
            Controls.Add(l);
        }

        void btnDeactivate_Click(object sender, EventArgs e)
        {
            Label l = new Label();
            l.Text = "Deactivating Group: " + radioButtons.SelectedItem.Value + ": ";

            foreach (SPFeatureDefinition definition in FindFeaturesByGroup(radioButtons.SelectedItem.Value))
            {
                if (CurrentWeb.Features.Contains(definition.Id))
                {
                    CurrentWeb.Features.Remove(definition.Id);
                    l.Text += definition.DisplayName + ",";
                }
            }
            Controls.Add(l);
        }


        IEnumerable<SPFeatureDefinition> FindFeaturesByGroup(string groupName)
        {
            // Features with a specific Group property
            var features = from SPFeatureDefinition f in CurrentFarm.FeatureDefinitions
                           where f.Scope == SPFeatureScope.Web
                           && (from SPFeatureProperty prop in f.Properties
                               where prop.Name == "Group" && prop.Value == groupName
                               select prop).Count() > 0
                           select f;

            return features;
        }



        

    }
}
