﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace JustAddCode.EffectiveFeatures
{
    public class FeatureLifecycle : Microsoft.SharePoint.SPFeatureReceiver
    {

        public FeatureLifecycle()
        {
            Debug.WriteLine("FeatureLifecycle ctor");
        }

        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            try
            {
                //System.Diagnostics.Debugger.Launch();
                Debug.WriteLine(
                    string.Format("{0} {1} {2} FeatureActivated", properties.Feature.Definition.DisplayName, GetParentType(properties), GetParentUrl(properties)));
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Exception " + ex.Message + " - " + ex.StackTrace);
            }
        }

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            try
            {
                Debug.WriteLine(
                    string.Format("{0} {1} {2} FeatureDeactivating", properties.Feature.Definition.DisplayName, GetParentType(properties), GetParentUrl(properties)));
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Exception " + ex.Message + " - " + ex.StackTrace);
            }
        }

        public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        {
            try
            {
                //System.Diagnostics.Debugger.Launch();
                Debug.WriteLine(
                    string.Format("{0} FeatureInstalled", properties.Definition.DisplayName));
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Exception " + ex.Message + " - " + ex.StackTrace);
            }
        }

        public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        {
            try
            {
                Debug.WriteLine(
                    string.Format("{0} {1} FeatureUninstalling", properties.Definition.DisplayName, SPFarm.Local.Name));
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Exception " + ex.Message + " - " + ex.StackTrace);
            }
        }

        private string GetParentType(SPFeatureReceiverProperties properties)
        {
            string parentType = "Unknown";
            if (typeof(SPFarm) == properties.Feature.Parent.GetType())
            { parentType = "Farm"; }
            else if (typeof(SPWebApplication) == properties.Feature.Parent.GetType())
            { parentType = "WebApp"; }
            else if (typeof(SPSite) == properties.Feature.Parent.GetType())
            { parentType = "Site"; }
            else if (typeof(SPWeb) == properties.Feature.Parent.GetType())
            { parentType = "Web"; }
            return parentType;
        }

        private string GetParentUrl(SPFeatureReceiverProperties properties)
        {
            string parentType = "Unknown";
            if (typeof(SPFarm) == properties.Feature.Parent.GetType())
            { parentType = ((SPFarm)properties.Feature.Parent).Name; }
            else if (typeof(SPWebApplication) == properties.Feature.Parent.GetType())
            { parentType = ((SPWebApplication)properties.Feature.Parent).Name; }
            else if (typeof(SPSite) == properties.Feature.Parent.GetType())
            { parentType = ((SPSite)properties.Feature.Parent).Url; }
            else if (typeof(SPWeb) == properties.Feature.Parent.GetType())
            { parentType = ((SPWeb)properties.Feature.Parent).Url; }
            return parentType;
        }


    }
}
