﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace JustAddCode.EffectiveFeatures
{
    public class ProjectLifecycleWebPart : WebPart
    {
        public enum ProjectLifecycle
        {
            Requirements = 1,
            Development = 2,
            Support = 3
        } ;

        private Label requirementsPhase = null;
        private Label devPhase = null;
        private Label supportPhase = null;
        private Button btnAdvanceStage = null;

        private SPFarm CurrentFarm
        {
            get
            {
                return Microsoft.SharePoint.Administration.SPFarm.Local;
            }
        }

        private SPWeb CurrentWeb
        {
            get
            {
                return SPContext.Current.Web;
            }
        }

        private ProjectLifecycle _currentPhase = ProjectLifecycle.Requirements;
        private ProjectLifecycle CurrentPhase
        {
            get
            {
                if (CurrentWeb.Properties.ContainsKey("ProjectLifecycle"))
                {
                    try
                    {
                        _currentPhase = (ProjectLifecycle)Enum.Parse(typeof(ProjectLifecycle), CurrentWeb.Properties["ProjectLifecycle"]);
                    }
                    catch (Exception)
                    {
                        IntializeProject();
                    }
                }
                else
                    IntializeProject();

                return _currentPhase;
            }
            set
            {
                _currentPhase = value;
                CurrentWeb.Properties["ProjectLifecycle"] = _currentPhase.ToString();
                CurrentWeb.Properties.Update();
            }
        }


        protected override void CreateChildControls()
        {
            base.CreateChildControls();

            DisplayCurrentPhase();

            btnAdvanceStage = new Button();
            btnAdvanceStage.Text = "Next Stage";
            btnAdvanceStage.Click += new EventHandler(btnAdvanceStage_Click);
            Controls.Add(btnAdvanceStage);
            Controls.Add(new Literal() { Text = "<BR>" });
            
        }

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);
            DisplayCurrentPhase();

            Controls.Add(requirementsPhase);
            Controls.Add(new Literal() { Text = "<BR>" });
            Controls.Add(devPhase);
            Controls.Add(new Literal() { Text = "<BR>" });
            Controls.Add(supportPhase);
            Controls.Add(new Literal() { Text = "<BR>" });

            if (CurrentPhase == ProjectLifecycle.Support)
                btnAdvanceStage.Visible = false;
        }

        private void DisplayCurrentPhase()
        {
            requirementsPhase = new Label() {Text = "1. Requirements Phase"};
            devPhase = new Label() {Text = "2. Development Phase"};
            supportPhase = new Label() {Text = "3. Support Phase"};

            switch(CurrentPhase)
            {
                case ProjectLifecycle.Requirements:
                    requirementsPhase.CssClass = "UserShadedRegion";
                    devPhase.CssClass = "UserNotification";
                    supportPhase.CssClass = "UserNotification";
                    break;
                case ProjectLifecycle.Development:
                    requirementsPhase.CssClass = "UserConfiguration";
                    devPhase.CssClass = "UserShadedRegion";
                    supportPhase.CssClass = "UserNotification";
                    break;
                case ProjectLifecycle.Support:
                    requirementsPhase.CssClass = "UserConfiguration";
                    devPhase.CssClass = "UserConfiguration";
                    supportPhase.CssClass = "UserShadedRegion";
                    break;
            }
               
        }

        void btnAdvanceStage_Click(object sender, EventArgs e)
        {
            CurrentPhase++;
            AdvanceStage(CurrentPhase);
            DisplayCurrentPhase();
        }

        void AdvanceStage(ProjectLifecycle newLifecycle)
        {
            // Features with a Group property
            var features = from SPFeatureDefinition f in CurrentFarm.FeatureDefinitions
                           where f.Scope == SPFeatureScope.Web
                           && (from SPFeatureProperty prop in f.Properties
                               where prop.Name == "ProjectLifecycle" && prop.Value==Enum.GetName(typeof(ProjectLifecycle),newLifecycle)
                               select prop).Count() > 0
                           select f;

            foreach (SPFeatureDefinition definition in features)
            {
                if (!CurrentWeb.Features.Contains(definition.Id))
                {
                    CurrentWeb.Features.Add(definition.Id);
                }
            }
        }

        void IntializeProject()
        {
            CurrentWeb.AllowUnsafeUpdates = true;   // Initial Request comes in as a GET, make sure we can update our SPWeb
            CurrentPhase = ProjectLifecycle.Requirements;
            AdvanceStage(_currentPhase);    // Activate the Phase since its our first time
        }

    }
}
